<?php

require_once(__DIR__.'/_config.php');

ini_set('display_errors', 'On');
error_reporting(E_ALL);

function sanitise($str) {
	return preg_replace('~[^a-zA-Z0-9_]~', '_', $str);
}

function hesc($str) {
	return htmlentities($str, ENT_QUOTES, 'UTF-8');
}

function bb2html($str) {
	return preg_replace_callback('~\\[url=(.*)\\](.*?)\\[/url\\]~', function($match) {
		return '<a href="'.$match[1].'">'.$match[2].'</a>';
	}, hesc($str));
}

function init_repo() {
	shell_exec(
		'mkdir '.escapeshellarg(REPO_DIR).' && cd '.escapeshellarg(REPO_DIR).' && '.
		'hg init && touch index && hg add index && hg commit -m init -u hgwiki'
	);
}

if (! is_dir(REPO_DIR)) {
	init_repo();
}

$name = '';
$currentVersion = 0;
$allowEdit = false;
$allowPageControls = true;

switch ($_GET['p']) {
	
	case 'set': {
		$name = sanitise($_POST['name']);
		$body = $_POST['body'];
		file_put_contents(REPO_DIR.$name, $body);
		shell_exec('cd '.REPO_DIR.' ; hg add '.escapeshellarg($name).' ; hg commit -m '.escapeshellarg($name).' -u hgwiki');
		header('Location: ?p=get&name='.urlencode($name));
		die();
	} break;
	
	case 'get': {
		$name = sanitise($_GET['name']);
		if (array_key_exists('edit', $_GET)) {
			$allowEdit = true;
		}
		if (file_exists(REPO_DIR.$name)) {
			$body = file_get_contents(REPO_DIR.$name);
		} else {
			die('Not found');
		}
	} break;
	
	case 'list': {
		$allowPageControls = false;
		$body = shell_exec('cd '.REPO_DIR.' ; ls | sort | sed -re \'s~(.+)~[url=?p=get\&name=\1]\1[/url]~\'');
	} break;
	
	case 'rev': {
		$name = sanitise($_GET['name']);
		$currentVersion = intval($_GET['ver']);
		$body = shell_exec('cd '.REPO_DIR.' ; hg cat '.escapeshellarg($name).' -r '.escapeshellarg($currentVersion));
	} break;
	
	case 'diff': {
		$name = sanitise($_GET['name']);
		$from = intval($_GET['from']);
		$to = intval($_GET['to']);
		$body = shell_exec('cd '.REPO_DIR.' ; hg diff -r '.escapeshellarg($from).' -r '.escapeshellarg($to).' '.escapeshellarg($name));
	} break;
	
	case 'hist': {
		$name = sanitise($_GET['name']);
		if (file_exists(REPO_DIR.$name)) {
			$body = shell_exec('cd '.REPO_DIR.' ; hg log '.escapeshellarg($name));
		} else {
			die('Not found');
		}
	} break;
	
	default: {
		header('Location: ?p=get&name=index');
		die();
	} break;
	
}
?>
<!DOCTYPE html>
<html>
	<head>
		<title><?=hesc(PAGE_TITLE)?></title>
		<style type="text/css">
html,body {
	padding:0;
	margin:0;
	font-family:"Segoe UI",sans-serif;
}
form[method=GET] {
	display:inline;
}
h2 {
	margin:4px;
}
textarea {
	box-sizing:border-box;
	width:100%;
	min-height:300px;
	border:0;
}
input[type=number] {
	max-width:100px;
}
.menubar {
	background:linear-gradient(#FFF, #DDD);
	padding:4px;
	margin-bottom:4px;
	color:white;
}
.pagebody {
	white-space:pre;
	padding-left:4px;
}
		</style>
	</head>
	<body>
		<h2><?=hesc(PAGE_TITLE)?> <?=strlen($name) ? '&rsaquo; '.hesc($name) : ''?></h2>
		
		<div class="menubar">
			
			<form method="GET">
				<input type="hidden" name="p" value="list">
				<input type="submit" value="list">
			</form>
<?php if ($allowPageControls) { ?> 
			|
			<form method="GET">
				<input type="hidden" name="p" value="get">
				<input type="hidden" name="name" value="<?=hesc($name)?>">
				<input type="submit" value="latest">
			</form>
			|
			<form method="GET">
				<input type="hidden" name="p" value="get">
				<input type="hidden" name="edit" value="1">
				<input type="hidden" name="name" value="<?=hesc($name)?>">
				<input type="submit" value="edit">
			</form>
			|
			<form method="GET">
				<input type="hidden" name="p" value="hist">
				<input type="hidden" name="name" value="<?=hesc($name)?>">
				<input type="submit" value="history">
			</form>
			|
			<form method="GET">
				<input type="hidden" name="p" value="rev">
				<input type="hidden" name="name" value="<?=hesc($name)?>">
				<input type="number" name="ver" min="0" placeholder="View at rev" <?=($currentVersion > 0) ? 'value="'.$currentVersion.'"' : ''?>>
				<input type="submit" value="&raquo;">
			</form>
			|
			<form method="GET">
				<input type="hidden" name="p" value="diff">
				<input type="hidden" name="name" value="<?=hesc($name)?>">
				<input type="number" name="from" min="0" placeholder="Compare rev">
				<input type="number" name="to" min="0" placeholder="">
				<input type="submit" value="&raquo;">
			</form>
<?php } ?> 
		</div>
		
<?php if (! $allowEdit) { ?> 
		<div class="pagebody"><?=bb2html($body)?></div>
		
<?php } else { ?> 
		<form method="POST" action="?p=set">
			<textarea name="body"><?=hesc($body)?></textarea>
			
			<div class="menubar">
				<input type="text" name="name" value="<?=hesc($name)?>">
				<input type="submit" value="Save changes &raquo;">
			</div>
		</form>
<?php } ?> 
	</body>
</html>